import json
import os,sys,shutil
from PIL import Image
#from aicloud.voc_xml_generator import xml_fill
import yaml
import codecs
def find_image_size(filename):
    with Image.open(filename) as img:
        img_width = img.size[0]
        img_height = img.size[1]
        img_mode = img.mode
        if img_mode == "RGB":
            img_depth = 3
        elif img_mode == "RGBA":
            img_depth = 3
        elif img_mode == "L":
            img_depth = 1
        else:
            print("img_mode = %s is neither RGB or L" % img_mode)
            eixt(0)

        return img_width, img_height, img_depth


def json2voc(yamlPath):
    yamlPath = yamlPath
    f = open(yamlPath,'r',encoding='utf=8')
    cfg = f.read()
    cfgFile = yaml.load(cfg)
    #创建所需文件夹
    tt100k_parent_dir = cfgFile['OUTPUT_DIR']
    work_sapce_dir = os.path.join(tt100k_parent_dir, "VOCdevkit/")
    if not os.path.isdir(work_sapce_dir):
        os.mkdir(work_sapce_dir)
    work_sapce_dir = os.path.join(work_sapce_dir, "VOC2007/")
    if not os.path.isdir(work_sapce_dir):
        os.mkdir(work_sapce_dir)
    jpeg_images_path = os.path.join(work_sapce_dir, 'JPEGImages')
    annotations_path = os.path.join(work_sapce_dir, 'Annotations')
    if not os.path.isdir(jpeg_images_path):
        os.mkdir(jpeg_images_path)
    if not os.path.isdir(annotations_path):
        os.mkdir(annotations_path)
    
    print('***************************数据集开始转换************************************')
    input_labels_dir = cfgFile['INPUT_LABELS_DIR']
    input_images_dir = cfgFile['INPUT_IMAGES_DIR']

    for root,dirs, files in os.walk(input_labels_dir):
        for f in files:
            print('正在转换文件',f)
            json_name = f.split(".")[0]
            #读取标注信息并写入xml
            with open(input_labels_dir+json_name +'.json', 'r',encoding='utf-8') as f:
                object_json = json.load(f)
                #打开对应图像文件
                filename = input_images_dir + json_name + ".jpg"
                width,height,channels = find_image_size(filename)
                with codecs.open(annotations_path + '/' + json_name + ".xml", "w", "utf-8") as xml:
                    xml.write('<annotation>\n')
                    xml.write('\t<folder>' + 'VOC2007' + '</folder>\n')
                    xml.write('\t<filename>' + json_name + ".jpg" + '</filename>\n')
                    xml.write('\t<source>\n')
                    xml.write('\t\t<database>The UAV autolanding</database>\n')
                    xml.write('\t\t<annotation>UAV AutoLanding</annotation>\n')
                    xml.write('\t\t<image>flickr</image>\n')
                    xml.write('\t\t<flickrid>NULL</flickrid>\n')
                    xml.write('\t</source>\n')
                    xml.write('\t<owner>\n')
                    xml.write('\t\t<flickrid>NULL</flickrid>\n')
                    xml.write('\t\t<name>NingWang</name>\n')
                    xml.write('\t</owner>\n')
                    xml.write('\t<size>\n')
                    xml.write('\t\t<width>'+ str(width) + '</width>\n')
                    xml.write('\t\t<height>'+ str(height) + '</height>\n')
                    xml.write('\t\t<depth>' + str(channels) + '</depth>\n')
                    xml.write('\t</size>\n')
                    xml.write('\t\t<segmented>0</segmented>\n')


                    num_obj=len(object_json['objects'])
                    for i in range(num_obj):
                        obj = object_json['objects'][i]
                        label = str(obj['f_code'])
                        for i in range(len(obj['obj_points'])):
                            xmin=int(obj['obj_points'][i]['x'])
                            ymin=int(obj['obj_points'][i]['y'])
                            xmax= xmin + int(obj['obj_points'][i]['w'])
                            ymax= ymin + int(obj['obj_points'][i]['h'])
                            if xmax <= xmin:
                                pass
                            elif ymax <= ymin:
                                pass
                            else:
                                xml.write('\t<object>\n')
                                xml.write('\t\t<name>'+ str(label) + '</name>\n')
                                xml.write('\t\t<pose>Unspecified</pose>\n')
                                xml.write('\t\t<truncated>1</truncated>\n')
                                xml.write('\t\t<difficult>0</difficult>\n')
                                xml.write('\t\t<difficult>0</difficult>\n')
                                xml.write('\t\t<bndbox>\n')
                                xml.write('\t\t\t<xmin>' + str(xmin) + '</xmin>\n')
                                xml.write('\t\t\t<ymin>' + str(ymin) + '</ymin>\n')
                                xml.write('\t\t\t<xmax>' + str(xmax) + '</xmax>\n')
                                xml.write('\t\t\t<ymax>' + str(ymax) + '</ymax>\n')
                                xml.write('\t\t</bndbox>\n')
                                xml.write('\t</object>\n')
                                #print(json_name,xmin,ymin,xmax,ymax,label)
                    xml.write('</annotation>')

    return 



import os
import base64
import time
import sys
import traceback
import logging
import json
import requests
from aicloud.utils import RedisClient
import io
import importlib
import aicloud.settings as settings

# root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
root = os.path.abspath(os.path.dirname(os.getcwd()))
sys.path.append(root)
logging.basicConfig(level=logging.DEBUG, format="%(asctime)s %(name)s:%(levelname)s:%(message)s",
                    datefmt="%Y-%M-%d %H:%M:%S")


def __insert_redis(param_path, src_redis):
    images_path = os.listdir(param_path)
    images_dict = {}
    count = 0
    # 10 images are default
    for image_path in images_path:
        try:
            if count < 10:
                absolute_image_path = os.path.join(param_path, image_path)
                logging.info(
                    "__insert_redis image path:{0} count:{1} ".format(absolute_image_path, count))
                with open(absolute_image_path, 'rb') as f:
                    images_dict[image_path] = f.read()
                count += 1
        except Exception:
            logging.error(traceback.format_exc())
    now_time_key = time.strftime("%Y%m%d_%H%M%S", time.localtime())
    src_batch_id = now_time_key + "_debug_image"
    src_redis.hmset(src_batch_id, images_dict)
    return src_batch_id


def simple_inference(infer_rlt_redis_dict=settings.DEBUG_INFER_RLT_REDIS,
                     is_debug=True, module_path=None, images_path=None):
    logging.info(
        'simple_inference input0 infer_rlt_redis_dict: {0} is_debug: {1} module_path: {2} images_path: {3}'.format(
            infer_rlt_redis_dict, is_debug, module_path, images_path))
    # defaults
    debug_src_redis_dict = settings.DEBUG_SRC_REDIS_DICT
    engine_class_name = settings.ENGINE_CLASS_NAME
    load_model_method_name = settings.LOAD_MODEL_METHOD_NAME
    inference_method_name = settings.INFERENCE_METHOD_NAME
    release_method_name = settings.RELEASE_GPU_RES_METHOD_NAME

    if is_debug:
        if images_path:
            # debug algorithm in notebook
            src_redis = RedisClient(debug_src_redis_dict["redis_host"], debug_src_redis_dict["password"],
                                    debug_src_redis_dict["db"])
            src_batch_id = __insert_redis(images_path, src_redis)
            result_key = src_batch_id + '_result_key'
            progress_key = src_batch_id + '_progress_key'
            result_redis = RedisClient(infer_rlt_redis_dict["redis_host"], infer_rlt_redis_dict["password"],
                                       infer_rlt_redis_dict["db"])
        else:
            logging.error('there is no images_path in debug mode')
            return
    else:
        src_batch_id = settings.SRC_BATCH_ID
        result_key = settings.RESULT_KEY
        progress_key = settings.PROGRESS_KEY
        src_redis = RedisClient(settings.SRC_REDIS_HOSTS, settings.SRC_REDIS_PASSWORD, settings.SRC_REDIS_DB)
        result_redis = RedisClient(settings.DST_REDIS_HOSTS, settings.DST_REDIS_PASSWORD, settings.DST_REDIS_DB)

    if not module_path:
        logging.error("no module_path")
        return
    else:
        alg_module = importlib.import_module(module_path)

    infer_engine = getattr(alg_module, engine_class_name)()
    logging.info('__init__ method  executed')
    load_model_func = getattr(infer_engine, load_model_method_name)
    load_model_func()
    logging.info('load_model method executed')

    count = 0
    if is_debug or settings.INFERENCE_TYPE == '1':
        # recoder progress count
        key = src_redis.hkeys(src_batch_id)
        if len(key) == 0:
            logging.error('no source data found')
        now_time_str = time.strftime("%Y%m%d_%H%M%S", time.localtime())
        for i in range(len(key)):
            logging.info('batch inference start count is:' + str(count))
            # read image from redis and encode
            data = src_redis.hget(src_batch_id, key[i])
            base64_data = base64.b64encode(data).decode()
            result_json = __one_pic_inference__(infer_engine, inference_method_name, base64_data)
            # store inference result and progress in redis
            # we will not process exception about the redis in order to restart automatically
            count = count + 1
            result_redis.hset(result_key, str(key[i], encoding="utf-8"), json.dumps(result_json))
            result_redis.set(progress_key, count)
            if is_debug:
                # write debug result
                result_file_full_name = os.getcwd() + "/" + now_time_str + "_debug_result"
                with open(result_file_full_name, 'a+') as f:
                    f.write(json.dumps(result_json) + "\n\n")
                logging.info('your debug result has been written to' + result_file_full_name)
    elif settings.INFERENCE_TYPE == '0':
        # single pic inference
        logging.info('batch inference single pic start ')
        base64_data = src_redis.get(src_batch_id)
        result_json = __one_pic_inference__(infer_engine, inference_method_name, base64_data)
        result_redis.set(result_key, json.dumps(result_json))
        count = count + 1

    # post process
    if is_debug:
        src_redis.delete(src_batch_id)
        result_redis.expire(result_key, 3600)
    else:
        # release GPU quotas
        try:
            release_func = getattr(infer_engine, release_method_name)
            release_func()
            tmpUrl = os.getenv("releaseQuotaUrl")
            url = tmpUrl + str(count)
            logging.info('releaseQuotaUrl url:{}'.format(url))
            r = requests.get(url)
            msg = 'ai successfully' if r.status_code == 200 else 'ai failed'
            logging.info(msg)
        except:
            logging.error(traceback.format_exc())


def __one_pic_inference__(infer_engine, inference_method_name, base64_data):
    result_json = []
    try:
        t1 = time.time()
        infer_func = getattr(infer_engine, inference_method_name)
        result = infer_func(base64_data)
        t2 = time.time()
        infer_time = t2 - t1
        result_json = json.loads(result)
        # add inference time
        result_json['infer_time'] = infer_time
    except:
        logging.error(traceback.format_exc())
        result_json['msg'] = traceback.format_exc()
    return result_json


if __name__ == '__main__':
    simple_inference(is_debug=False, module_path=settings.ENGINE_MODULE_PATH)


import logging
from logging.handlers import TimedRotatingFileHandler
from kafka import KafkaProducer, KafkaConsumer


class RedisClient(object):
    def __init__(self, hosts, password=None, db=0):
        redis_hosts = self.parse_redis_hosts(hosts)
        if redis_hosts.__len__() is 0:
            self.rclient = None
        elif redis_hosts.__len__() is 1:
            import redis
            self.rclient = redis.StrictRedis(password=password,
                                             db=db,
                                             **redis_hosts[0])
        else:
            import rediscluster
            self.rclient = rediscluster.RedisCluster(password=password, startup_nodes=redis_hosts)

    @staticmethod
    def parse_redis_hosts(hosts):
        redis_hosts = []
        if isinstance(hosts, str):
            hosts = hosts.split(':')
            redis_hosts.append({'host': hosts[0],
                                'port': hosts[1]})
            return redis_hosts

        for host in hosts:
            address = host.split(':')
            redis_hosts.append({'host': address[0],
                                'port': address[1]})

        return redis_hosts

    def set(self, key, value, seconds=None):
        return self.rclient.set(key, value, px=seconds)

    def get(self, key):
        return self.rclient.get(key)

    def hkeys(self, name):
        return self.rclient.hkeys(name)

    def hset(self, name, key, value):
        return self.rclient.hset(name, key, value)

    def hget(self, name, key):
        return self.rclient.hget(name, key)

    def delete(self, key):
        return self.rclient.delete(key)

    def hmset(self, name, mapping):
        return self.rclient.hmset(name, mapping)

    def expire(self, key, time):
        return self.rclient.expire(key, time)



class KafkaClient(object):
    def __init__(self, host, username=None, password=None):
        self.producer = KafkaProducer(bootstrap_servers=host,
                                      sasl_plain_username=username,
                                      sasl_plain_password=password)
        self.consumer = KafkaConsumer(bootstrap_servers=host,
                                      sasl_plain_username=username,
                                      sasl_plain_password=password)

    def send(self, topic, value):
        return self.producer.send(topic=topic, value=value)

    def p_close(self):
        return self.producer.close()

    def subscribe(self, topics):
        return self.consumer.subscribe(topics=topics)

    def poll(self):
        return self.consumer.poll()

    def c_close(self):
        return self.consumer.close()

    def close_all(self):
        self.producer.close()
        self.consumer.close()
        return 0


# log
def log():
    handler = TimedRotatingFileHandler("inference.log", when="D", interval=2, backupCount=5,
                                       encoding="UTF-8", delay=False, utc=True)
    logging_format = logging.Formatter(
        '%(asctime)s - %(levelname)s - %(filename)s - %(funcName)s - %(lineno)s - %(message)s')
    handler.setFormatter(logging_format)
    return handler


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time           : 20-4-28 下午1:07
# @Author         : Shen Bin
# @File           : sample.py
# @Product        : PyCharm
# @Docs           :
# @Source         :

from aicloud.engine import AlgorithmEngine

ae = AlgorithmEngine()
#download model from cloud
#code, msg = ae.download_model('.')

# upload model to cloud
#msg = ae.upload_model('./test.txt')

#print(code,msg)
#create training task
#msg = ae.create_training_task('training_name', '/path/log2')
#print(msg)
#create training duration
msg = ae.create_training_duration()
print( msg)






# -*- coding: utf-8 -*-
import os
from os import path
import json
import numpy as np
import pandas as pd
import glob
import cv2
import os
import yaml
import shutil
from IPython import embed
from sklearn.model_selection import train_test_split
np.random.seed(41)


class Csv2CoCo:

    def __init__(self,image_dir,total_annos, classname_to_id):
        self.images = []
        self.annotations = []
        self.categories = []
        self.img_id = 0
        self.ann_id = 0
        self.image_dir = image_dir
        self.total_annos = total_annos
        self.classname_to_id = classname_to_id

    def save_coco_json(self, instance, save_path):
        json.dump(instance, open(save_path, 'w'), ensure_ascii=False, indent=2)  # indent=2 更加美观显示

    # 由txt文件构建COCO
    def to_coco(self, keys, classname_to_id):
        self._init_categories(classname_to_id)
        for key in keys:
            self.images.append(self._image(key))
            shapes = self.total_annos[key]
            for shape in shapes:
                bboxi = []
                for cor in shape[:-1]:
                    bboxi.append(int(cor))
                label = shape[-1]
                # if label != '16007' or label != '16008':
                #     continue
                annotation = self._annotation(bboxi,label, classname_to_id)
                self.annotations.append(annotation)
                self.ann_id += 1
            self.img_id += 1
        instance = {}
        instance['info'] = 'spytensor created'
        instance['license'] = ['license']
        instance['images'] = self.images
        instance['annotations'] = self.annotations
        instance['categories'] = self.categories
        return instance

    # 构建类别
    def _init_categories(self, classname_to_id):
        for k, v in classname_to_id.items():
            category = {}
            category['id'] = v
            category['name'] = k
            self.categories.append(category)

    # 构建COCO的image字段
    def _image(self, path):
        image = {}
        #print(self.image_dir + path)
        print('正在转换:', path)
        img = cv2.imread(self.image_dir + path)
        image['height'] = img.shape[0]
        image['width'] = img.shape[1]
        image['id'] = self.img_id
        image['file_name'] = path
        return image

    # 构建COCO的annotation字段
    def _annotation(self, shape,label, classname_to_id):
        # label = shape[-1]
        points = shape[:4]
        annotation = {}
        annotation['id'] = self.ann_id
        annotation['image_id'] = self.img_id
        annotation['category_id'] = int(classname_to_id[label])
        annotation['segmentation'] = self._get_seg(points)
        annotation['bbox'] = self._get_box(points)
        annotation['iscrowd'] = 0
        annotation['area'] = self._get_area(points)
        return annotation

    # COCO的格式： [x1,y1,w,h] 对应COCO的bbox格式
    def _get_box(self, points):
        min_x = points[0]
        min_y = points[1]
        max_x = points[2]
        max_y = points[3]
        return [min_x, min_y, max_x - min_x, max_y - min_y]
    # 计算面积
    def _get_area(self, points):
        min_x = points[0]
        min_y = points[1]
        max_x = points[2]
        max_y = points[3]
        return (max_x - min_x+1) * (max_y - min_y+1)
    # segmentation
    def _get_seg(self, points):
        min_x = points[0]
        min_y = points[1]
        max_x = points[2]
        max_y = points[3]
        h = max_y - min_y
        w = max_x - min_x
        a = []
        a.append([min_x,min_y, min_x,min_y+0.5*h, min_x,max_y, min_x+0.5*w,max_y, max_x,max_y, max_x,max_y-0.5*h, max_x,min_y, max_x-0.5*w,min_y])
        return a
   


def json_to_csv(path, img_path):
    json_list = []
    for json_file in glob.glob(path + '/*.json'):  #返回所有匹配的文件路径列表
        fp_json_file = json.load(open(json_file, "r"))
        filename = os.path.basename(json_file)
        for multi in fp_json_file["objects"]:
            points = multi["obj_points"][0]
            xmin = points["x"]
            ymin = points["y"]
            xmax = points["x"]+points["w"]-1
            ymax = points["y"]+points["h"]-1
            label = multi["f_code"]
            if xmax <= xmin: pass
            elif ymax <= ymin: pass
            value = (img_path +'/' + filename[:-5] + '.jpg',
                    int(xmin),
                    int(ymin),
                    int(xmax),
                    int(ymax),
                    str(label))
            json_list.append(value)
    column_name = ['filename', 'xmin', 'ymin', 'xmax', 'ymax', 'class']
    xml_df = pd.DataFrame(json_list, columns=column_name)
                    
    return xml_df



def json2coco(yamlPath):
    yamlPath = yamlPath
    f = open(yamlPath,'r',encoding='utf=8')
    cfg = f.read()
    cfgFile = yaml.load(cfg)
 
    img_path = cfgFile['INPUT_IMAGES_DIR']
    print('输入图像路径:',  img_path)
    output_path = cfgFile['OUTPUT_DIR']
    print('输出结果路径:', output_path)
    #print('数据集开始转换')
    xml_df = json_to_csv(cfgFile['INPUT_LABELS_DIR'], img_path)
    ## 修改文件名称
    xml_df.to_csv('./scratches.csv', index=None)
    #print('Successfully converted xml to csv.')

    # 标注路径
    label_path = cfgFile['INPUT_LABELS_DIR']
    print('输入标注路径:', label_path)
    csv_file= "./scratches.csv"
    classname_to_id =  cfgFile['CLASS_ID']
    print('对象ID:', classname_to_id)
    print('********************数据集开始转换*************************')
    # 整合csv格式标注文件
    total_csv_annotations = {}
    annotations = pd.read_csv(csv_file,header=None).values
    for annotation in annotations:
        key = annotation[0].split(os.sep)[-1]
        value = np.array([annotation[1:]])
        if key in total_csv_annotations.keys():
            total_csv_annotations[key] = np.concatenate((total_csv_annotations[key],value),axis=0)
        else:
            total_csv_annotations[key] = value
    # 按照键值划分数据
    total_keys = list(total_csv_annotations.keys())
    train_keys, val_keys = train_test_split(total_keys[1:], test_size=0.2) #去掉第一行
    print('*******************训练集和验证集划分**********************')
    print("训练集数目:", len(train_keys), '验证集数目:', len(val_keys))
    # 创建必须的文件夹
    #print('转换后数据保存在datasets目录')

    if not os.path.exists(output_path + 'datasets/annotations/ark/'):
        os.makedirs(output_path + 'datasets/annotations/ark/')
    if not os.path.exists(output_path+ 'datasets/train/'):
        os.makedirs(output_path + 'datasets/train/')
    if not os.path.exists(output_path + 'datasets/test/'):
        os.makedirs(output_path + 'datasets/test/')
    # 把训练集转化为COCO的json格式
    print('*********************训练集转换中，请稍后**************************')
    l2c_train = Csv2CoCo(image_dir=img_path, total_annos=total_csv_annotations, classname_to_id=classname_to_id)
    train_instance = l2c_train.to_coco(train_keys, classname_to_id)
    l2c_train.save_coco_json(train_instance, output_path + 'datasets/annotations/ark/instances_train2020.json')
    print('**********************训练集转换完成*******************************')
    print('***********************验证集转换中,请稍后*************************')
    #for file in train_keys:
    #    shutil.copy(img_path+file,output_path + "datasets/train/")
    #for file in val_keys:
    #    shutil.copy(img_path+file,output_path + "datasets/test/")
    # 把验证集转化为COCO的json格式
    l2c_val = Csv2CoCo(image_dir=img_path,total_annos=total_csv_annotations, classname_to_id=classname_to_id)
    val_instance = l2c_val.to_coco(val_keys, classname_to_id)
    l2c_val.save_coco_json(val_instance, output_path + 'datasets/annotations/ark/instances_val2020.json')
    print('************************验证集转换完成****************************')
    print('************************正在进行图片拷贝**************************')
    for file in train_keys:
        shutil.copy(img_path+file,output_path + "datasets/train/")
    for file in val_keys:
        shutil.copy(img_path+file,output_path + "datasets/test/")
    print('************************数据集转换完成****************************')

     




import json
import os,sys,shutil
from PIL import Image
from aicloud.voc_xml_generator import xml_fill
import yaml

def find_image_size(filename):
    with Image.open(filename) as img:
        img_width = img.size[0]
        img_height = img.size[1]
        img_mode = img.mode
        if img_mode == "RGB":
            img_depth = 3
        elif img_mode == "RGBA":
            img_depth = 3
        elif img_mode == "L":
            img_depth = 1
        else:
            print("img_mode = %s is neither RGB or L" % img_mode)
            eixt(0)

        return img_width, img_height, img_depth


def json2voc(yamlPath):
    yamlPath = yamlPath
    f = open(yamlPath,'r',encoding='utf=8')
    cfg = f.read()
    cfgFile = yaml.load(cfg)
    #创建所需文件夹
    tt100k_parent_dir = cfgFile['OUTPUT_DIR']
    work_sapce_dir = os.path.join(tt100k_parent_dir, "VOCdevkit/")
    if not os.path.isdir(work_sapce_dir):
        os.mkdir(work_sapce_dir)
    work_sapce_dir = os.path.join(work_sapce_dir, "VOC2007/")
    if not os.path.isdir(work_sapce_dir):
        os.mkdir(work_sapce_dir)
    jpeg_images_path = os.path.join(work_sapce_dir, 'JPEGImages')
    annotations_path = os.path.join(work_sapce_dir, 'Annotations')
    if not os.path.isdir(jpeg_images_path):
        os.mkdir(jpeg_images_path)
    if not os.path.isdir(annotations_path):
        os.mkdir(annotations_path)
    
    print('数据集开始转换')
    input_labels_dir = cfgFile['INPUT_LABELS_DIR']
    input_images_dir = cfgFile['INPUT_IMAGES_DIR']

    for root,dirs, files in os.walk(input_labels_dir):
        for f in files:
            print('正在转换文件',f)
            json_name = f.split(".")[0]
            with open(input_labels_dir+json_name +'.json', 'r',encoding='utf-8') as f:
                object_json = json.load(f)
                #打开对应图像文件
                filename = input_images_dir + json_name + ".jpg"
                width,height,depth = find_image_size(filename)
                filler = xml_fill(filename, width, height, depth)
                num_obj=len(object_json['objects'])
                for i in range(num_obj):
                    obj = object_json['objects'][i]
                    class_name = str(obj['f_code'])
                    for i in range(len(obj['obj_points'])):
                        xmin=int(obj['obj_points'][i]['x'])
                        ymin=int(obj['obj_points'][i]['y'])
                        xmax= xmin + int(obj['obj_points'][i]['w'])
                        ymax= ymin + int(obj['obj_points'][i]['h'])
                        filler.add_obj_box(class_name, xmin, ymin, xmax, ymax)
                filler.save_xml(annotations_path + '/' + json_name + '.xml')
                print("%s.xml saved\n"%json_name)
    return 



# -*- coding: utf-8 -*-
import os
import json

env_dist = os.environ
# async message storage by kafka
if 'status_provider' in env_dist:
    status_kafka = env_dist['status_provider']
    if status_kafka is None:
        print('Kafka status_provider does not exist!')
    else:
        status_kafka = json.loads(status_kafka)
        KAFKA_HOST = status_kafka['address'] if 'address' in status_kafka.keys() else None
        KAFKA_USERNAME = status_kafka['username'] if 'username' in status_kafka.keys() else None
        KAFKA_PASSWORD = status_kafka['password'] if 'password' in status_kafka.keys() else None
        KAFKA_TOPIC_STOP = status_kafka['topic'] if 'topic' in status_kafka.keys() else None
# data storage in Redis, the form of redis hosts and password like follow
# hosts = '10.70.151.183:6379' or
# ['10.70.151.183:6379', '10.70.151.225:6379', '10.70.151.205:6379',
# '10.70.151.232:6379', '10.70.151.239:6379', '10.70.151.231:6379']
# password = 'xxxxxxx'
if 'src_provider' in env_dist:
    src_redis = env_dist['src_provider']
    src_redis = json.loads(src_redis)
    SRC_REDIS_HOSTS = src_redis['address']
    SRC_REDIS_PASSWORD = src_redis['password'] if 'password' in src_redis.keys() else None
    SRC_REDIS_DB = src_redis['database'] if 'database' in src_redis.keys() else 0
    SRC_BATCH_ID = src_redis['batch_id']
if 'dst_provider' in env_dist:
    dst_redis = env_dist['dst_provider']
    dst_redis = json.loads(dst_redis)
    DST_REDIS_HOSTS = dst_redis['address']
    DST_REDIS_PASSWORD = dst_redis['password'] if 'password' in dst_redis.keys() else None
    DST_REDIS_DB = dst_redis['database'] if 'database' in dst_redis.keys() else 0
    RESULT_KEY = dst_redis['result_key']
    PROGRESS_KEY = dst_redis['progress_key']

if 'inference_type' in env_dist:
    INFERENCE_TYPE = env_dist['inference_type']
# DEBUG FINAL VALUES
DEBUG_INFER_RLT_REDIS = {"redis_host": "10.73.1.31:32730", "password": "ai13579", "db": "3"}
DEBUG_SRC_REDIS_DICT = {"redis_host": "10.73.1.31:32730", "password": "ai13579", "db": "1"}
ENGINE_MODULE_PATH = 'sdk.inference'
ENGINE_CLASS_NAME = 'Engine'
LOAD_MODEL_METHOD_NAME = 'load_model'
INFERENCE_METHOD_NAME = 'inference'
RELEASE_GPU_RES_METHOD_NAME = 'release_gpu_res'


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time           : 20-4-27 下午3:05
# @Author         : Shen Bin
# @File           : rest.py
# @Product        : PyCharm
# @Docs           :
# @Source         :

import requests

from aicloud.cloudexception import CommonError


class Object(dict):
    def __init__(self, init_dict=None, deep=False):
        super(Object, self).__init__()

        if isinstance(init_dict, dict):
            for k, v in init_dict.items():
                if deep:
                    if isinstance(v, dict):
                        if not isinstance(v, Object):
                            v = Object(v, deep)
                    elif isinstance(v, (list, tuple)):
                        v = [Object(i, deep) if (isinstance(i, dict) and not isinstance(i, Object)) else i for i in v]
                setattr(self, k, v)
                self[k] = v

    def __getattr__(self, key):
        pass

    def __getitem__(self, key):
        return getattr(self, key)


def _resp(r, no_error):
    if no_error and r.status_code >= 400:
        raise CommonError(code=r.status_code, message=r.content)

    return r.status_code, r.headers, r


def _resp_content(r):
    body = ""
    try:
        if r.text:
            body = r.json()
    except:
        raise CommonError(code=r.status_code, message=r.text)

    return r.status_code, r.headers, Object(body, True), r.content


def head(url, headers={}, verify=False, no_error=False, hooks=None):
    r = requests.get(url, headers=headers, verify=verify, hooks=hooks)
    return _resp(r, no_error)


def get(url, headers={}, verify=False, no_error=False, params=None, hooks=None):
    r = requests.get(url, headers=headers, verify=verify, params=params, hooks=hooks)
    return _resp(r, no_error)


def post(url, headers={}, body=None, verify=False, no_error=False, data=None, files=None, hooks=None):
    r = requests.post(url, headers=headers, json=body, data=data, verify=verify, files=files, hooks=hooks)
    return _resp(r, no_error)


def post_timeout(url, headers={}, body=None, verify=False, no_error=False, hooks=None):
    r = requests.post(url, headers=headers, json=body, verify=verify, timeout=8, hooks=hooks)
    return _resp(r, no_error)


def put(url, headers={}, body=None, verify=False, no_error=False, params=None, data=None, hooks=None):
    r = requests.put(url, headers=headers, json=body, params=params, data=data, verify=verify, hooks=hooks)
    return _resp(r, no_error)


def patch(url, headers={}, body=None, verify=False, no_error=False, hooks=None):
    r = requests.patch(url, headers=headers, json=body, verify=verify, hooks=hooks)
    return _resp(r, no_error)


def delete(url, headers={}, verify=False, no_error=False, hooks=None):
    r = requests.delete(url, headers=headers, verify=verify, hooks=hooks)
    return _resp(r, no_error)



import zipfile
import time
import os
import tarfile
import uuid

from aicloud.cloudapi import AIPlatform
import json
import sys
import io
#sys.stdout = sys.__stdout__ = io.TextIOWrapper(sys.stdout.detach(), encoding='utf-8', line_buffering=True)
#sys.stderr = sys.__stderr__ = io.TextIOWrapper(sys.stderr.detach(), encoding='utf-8', line_buffering=True)



class AlgorithmEngine(object):

    def __init__(self):
        self.token = os.environ.get('token')
        self.auth_token = os.environ.get('authToken')
        self.user_id = os.environ.get('userId')
        self.train_id = os.environ.get('trainId')
        self.base_url = os.environ.get('baseUrl')
        self.ai_platform = AIPlatform(base_url=self.base_url, authorization=self.token, auth_token=self.auth_token)
        self.nbid = os.environ.get('nbid')

    def upload_model(self, source_dir, target_dir=None):
        """
        upload model file to cloud storage
        :param source_dir: source directory of model files
        :param target_dir: target directory to zip model files
        :return:
        """
        target_file = target_dir + str(uuid.uuid1()) + ".tar" if target_dir else str(uuid.uuid1()) + ".tar"
        with tarfile.open(target_file, "w:") as tar:
            tar.add(source_dir, arcname=os.path.basename(source_dir))

        file = open(target_file, 'rb')
        code, header, body = self.ai_platform.upload_model_file(user_id=self.user_id,
                                                                train_id=self.train_id,
                                                                nbid=self.nbid,
                                                                model_file=file)
        #msg = 'upload model file successfully' if code == 200 else 'upload model file failed'
        if code is not 200:
            msg = 'upload model file failed'
            return code, msg 
        if json.loads(str(body.content, encoding = "utf-8"))['code'] == '0':
            msg = 'upload model file successfully'
        else:
             msg = json.loads(str(body.content, encoding = "utf-8"))
        # delete tar package
        file.close()
        os.remove(target_file)
        return msg

    def download_model(self, local_model_path):
        """
        download model file from cloud storage
        :param local_model_path: local storage path
        :return:
        """
        code, header, body = self.ai_platform.download_model_file(train_id=self.train_id)
        if code is not 200:
            msg = 'download model file failed'
            return code, msg
        if json.loads(str(body.content, encoding = "utf-8"))['code'] == '0':
            msg = 'download model file successfully'
        else:
             msg = json.loads(str(body.content, encoding = "utf-8"))
        #msg = 'download model file successfully'
        now_time = time.strftime("%Y-%m-%d-%H-%M-%S", time.localtime())
        file_absolute_path = local_model_path + '/' + now_time + '.tar'
        try:
            with open(file_absolute_path, "wb") as f:
                f.write(body.content)

            tar = tarfile.open(file_absolute_path)
            tar.extractall(local_model_path)
            tar.close()
        except Exception as ex:
            return 500, ex

        if os.path.exists(file_absolute_path):
            os.remove(file_absolute_path)

        return code, msg

    def create_training_task(self, name, log_path, description=None):
        """
        create training task, then can find in training list
        training task include name, description, version, dataset
        version is generated automatically associated with project, project is from env
        dataset is training dataset from env
        :param name: traitr(body.content, encoding = "utf-8")ing task name
        :param description: description of the task
        :return:
        """
        #test_url = self.base_url + '/ai/api/v1/training/training/form'
        code, header, body = self.ai_platform.save_model_info(name, log_path, self.nbid)
        if code is not 200:
            msg = 'create training task failed'
            return code, msg
        if json.loads(str(body.content, encoding = "utf-8"))['code'] == '0':
            msg = 'create training successfully'
            train_id = json.loads(str(body.content, encoding = "utf-8"))['result']['data']['trainingId']
            with open('/workspace/trainid.txt', 'w') as file:
                file.write(str(train_id))
        else:
            msg = json.loads(str(body.content, encoding = "utf-8"))
        return msg
    def create_training_duration(self):
        #url = self.base_url + '/ai/api/v1/training/finish/form/'
        code, header, body = self.ai_platform.training_duration_info()
        if code is not 200:
            msg = 'create training duration failed'
            return code, msg
        if json.loads(str(body.content, encoding = "utf-8"))['code'] == '0':
            msg = 'create training duration successfully'
        else:
             msg = json.loads(str(body.content, encoding = "utf-8"))
        return  msg







#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time           : 20-4-27 下午4:05
# @Author         : Shen Bin
# @File           : cloudapi.py
# @Product        : PyCharm
# @Docs           :
# @Source         :
from aicloud.rest import get, post, put
import json
import os

class AIPlatform(object):

    model_upload_path = '/model/file'
    model_download_path = '/model/download/modelfile'
    create_training_task = '/training/training/form'
    create_training_duration = '/training/finish/form/'    
    def __init__(self, base_url, version='v1', authorization=None, auth_token=None):
        self.authorization = authorization
        self.auth_token = auth_token
        self.url = base_url.rstrip('/') + '/ai/api/' + version
        self.train_id_file = '/workspace/trainid.txt'
    def _make_token_headers(self, content_type=None):
        headers = {
            'Authorization': self.authorization,
            'auth-token': self.auth_token
        }
        if content_type:
            headers['Content-Type'] = content_type

        return headers

    def upload_model_file(self, user_id, train_id, nbid, model_file):
        """
        upload model file to cloud storage
        :param user_id: user id
        :param train_id: training id
        :param model_file: model file by training
        :return:
        """
        if os.path.exists(self.train_id_file):
            with open(self.train_id_file) as f:
                train_id = f.readline()

        url = self.url + self.model_upload_path
        file = {'multipartFile': model_file}
     
        data = {
            'trainingId': train_id,
            'userId': user_id,
            'nbid': nbid
        }

        return post(url, headers=self._make_token_headers(), data=data, files=file)

    def download_model_file(self, train_id):
        """
        download model file from cloud storage
        :param train_id: training id
        :return:
        """
        url = self.url + self.model_download_path
        params = {'trainingId': train_id}

        return get(url, headers=self._make_token_headers(), params=params)
    def save_model_info(self, training_name, log_path,  nbid):
        """
        :param training_name:
        :param nbid:
        :param task_url:
        :return:
        """
        url = self.url + self.create_training_task
        data = {
            'trainingName': training_name,
            'notebookId': nbid,
            'logAddr': log_path
        }

        return post(url, headers=self._make_token_headers(content_type='application/json'), data=json.dumps(data))
    
    def training_duration_info(self):
        if os.path.exists(self.train_id_file):
            with open(self.train_id_file) as f:
                train_id = f.readline()
                url = self.url + self.create_training_duration + str(train_id)
        return put(url, headers=self._make_token_headers())     


class StorageEngine(object):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time           : 20-4-27 下午2:54
# @Author         : Shen Bin
# @File           : cloudexception.py
# @Product        : PyCharm
# @Docs           :
# @Source         :


class CommonError(Exception):
    code = 500
    error = "CommonError"
    message = "unknown error."

    def __init__(self, code=None, error=None, message=None):
        self.code = self.__class__.code if not code else code
        self.error = self.__class__.error if not error else error
        self.message = self.__class__.message if not message else message

    def __str__(self):
        return '<{}>: {}'.format(self.__class__.__name__, self.message)



import setuptools
setuptools.setup(
    name="NCloudAISDK",                                     # 包的分发名称，使用字母、数字、_、-
    version="1.0.1",                                        # 版本号
    author="ai",                                       # 作者名字
    author_email="example@example.com",                      # 作者邮箱
    description="NCloudAISDK",                            # 包的简介描述
    packages=setuptools.find_packages(),                    # 如果项目由多个文件组成，我们可以使用find_packages()自动发现所有包和子包，而不是手动列出每个包，在这种情况下，包列表将是example_pkg
    classifiers=[                                           # 关于包的其他元数据(metadata)
        "Programming Language :: Python :: 3",              # 该软件包仅与Python3兼容
        "License :: OSI Approved :: MIT License",           # 根据MIT许可证开源
        "Operating System :: OS Independent",               # 与操作系统无关
    ],
    install_requires=['kafka>=1.3.5', 'redis>=3.5.3'],  # 依赖的包
    python_requires='>=3')


#coding:utf-8
import os
import json
import time
from PIL import Image
import numpy as np
import cv2
import yaml


def json2mapillary(yamlPath):
    yamlPath = yamlPath
    f = open(yamlPath,'r',encoding='utf=8')
    cfg = f.read()
    cfgFile = yaml.load(cfg)
    print('***************************数据集开始转换*****************************')
    input_labels_dir = cfgFile['INPUT_LABELS_DIR']
    input_images_dir = cfgFile['INPUT_IMAGES_DIR']
    output_dir = cfgFile['OUTPUT_DIR']
    class_type  = cfgFile['CLASS_TYPE']

    for root,dirs, files in os.walk(input_labels_dir):
        for f in files:
            print('正在转换文件',f)
            json_name = f.split(".")[0]
            with open(input_labels_dir+json_name +'.json', 'r',encoding='utf-8') as f:
                object_json = json.load(f)
                num_obj=len(object_json['objects'])
                image = Image.open(input_images_dir+json_name +'.jpg')
                image = np.array(image)
                image_h, image_w = image.shape[:2]
                image_temp =np. zeros((image_h,image_w),dtype=np.uint8)
                
                #print('num_obj=',num_obj)
                for i in range(num_obj):
                    obj = object_json['objects'][i]
                    class_name = str(obj['f_code']).lower()
                    for item in class_type:
                        if class_name == str(item):
                            label_polygon = []
                            for i in range(len(obj['obj_points'])):
                                x=int(obj['obj_points'][i]['x'])
                                y=int(obj['obj_points'][i]['y'])
                                label_xy = [x,y]
                                label_polygon.append(label_xy)
                                a = np.array(label_polygon)
                                                       
                            cv2.fillPoly(image_temp, [a], class_type[item])
                            
                cv2.imwrite(output_dir + json_name +'.png', image_temp)
              
